from fastapi import FastAPI
from src.models import CompletionRequest, CompletionResponse
from src.text_generation import generate_text

app: FastAPI = FastAPI(
    title="UV Demo API",
)

# In-memory storage for demo
completions: list[CompletionResponse] = []

@app.post("/completions/", response_model=CompletionResponse)
async def create_completion(
    completion_request: CompletionRequest
) -> CompletionResponse:
    """Create a new completion."""
    print(completion_request)
    completion_response = CompletionResponse(
        id=len(completions) + 1,
        prompt=completion_request.prompt,
        response=generate_text(prompt=completion_request.prompt),
    )
    completions.append(completion_response)
    print(completion_response)
    return completion_response

@app.get("/completions/", response_model=list[CompletionResponse])
async def list_completions() -> list[CompletionResponse]:
    """List all completions."""
    return completions



from pydantic import BaseModel
from pydantic import Field

class CompletionRequest(BaseModel):
    """Schema for creating a new completion."""
    prompt: str = Field(..., min_length=1, max_length=200)

class CompletionResponse(CompletionRequest):
    """Schema for a completion with ID."""
    response: str = Field(..., min_length=1, max_length=500)
    id: int = Field(..., gt=0)



import pytest
from httpx import ASGITransport
from httpx import AsyncClient

from src.main import app

@pytest.mark.anyio
async def test_request_completion():
    # Arrange
    prompt = "Hello, World!"

    # Act
    async with AsyncClient(
        transport=ASGITransport(app=app),
        base_url="http://test",
    ) as client:
        response = await client.post(
            "/completions/",
            json={
                "prompt": prompt,
            }
        )

        # Assert
        assert response.status_code == 200
        data = response.json()
        assert data["prompt"] == prompt
        assert "id" in data

@pytest.mark.anyio
async def test_list_completions():
    # Act
    async with AsyncClient(
        transport=ASGITransport(app=app),
        base_url="http://test",
    ) as client:
        response = await client.get("/completions/")

        # Assert
        assert response.status_code == 200
        data = response.json()
        assert isinstance(data, list)



import torch
from tokenizers import Tokenizer
from tokenizers.models import Model
from torch.fx.experimental.symbolic_shapes import lru_cache
from transformers import AutoModelForCausalLM
from transformers import AutoTokenizer


@lru_cache(maxsize=1)
def setup_model_and_tokenizer() -> tuple[Model, Tokenizer]:
    # Initialize tokenizer and model
    tokenizer = AutoTokenizer.from_pretrained(
        # pretrained_model_name_or_path="microsoft/Phi-3-mini-4k-instruct",
        pretrained_model_name_or_path="Qwen/Qwen2.5-0.5B-Instruct",
    )
    model = AutoModelForCausalLM.from_pretrained(
        #pretrained_model_name_or_path="microsoft/Phi-3-mini-4k-instruct",
        pretrained_model_name_or_path="Qwen/Qwen2.5-0.5B-Instruct",
        device_map="auto",
        torch_dtype=torch.float16,
    )
    return model, tokenizer

def generate_text(
    *,
    prompt: str,
    max_length: int = 50,
) -> str:
    model: Model
    tokenizer: Tokenizer
    model, tokenizer = setup_model_and_tokenizer()

    # Tokenize input
    inputs = tokenizer(prompt, return_tensors="pt").to(model.device)

    # Generate response
    outputs = model.generate(
        **inputs,
        max_length=max_length,
        temperature=0.7,
        do_sample=True,
        pad_token_id=tokenizer.pad_token_id,
    )

    # Decode and return response
    response: str = tokenizer.decode(outputs[0], skip_special_tokens=True)
    return response


def main() -> None:
    prompt: str = "Explain quantum computing in simple terms:"
    response: str = generate_text(prompt=prompt)
    print(response)


if __name__ == '__main__':
    main()



from src.text_generation import generate_text


def test_generate_text():
    # Arrange
    prompt_0 = "Explain quantum computing in simple terms:"
    prompt_1 = "What is the meaning of life?"
    prompt_2 = "What is the best programming language?"

    # Act
    response_0 = generate_text(prompt=prompt_0)
    response_1 = generate_text(prompt=prompt_1)
    response_2 = generate_text(prompt=prompt_2)

    assert response_0 is not None
    assert response_0 != ""
    assert response_1 is not None
    assert response_1 != ""
    assert response_2 is not None
    assert response_2 != ""
